@@ -7,13 +7,12 @@ Centralized test case repository for the SystemVerilog backend.
 Author : Peitian Pan
 Date   : Dec 16, 2019
 """
-
 from os.path import dirname
 from textwrap import dedent
 
-import pymtl3.passes.rtlir.rtype.RTLIRDataType as rdt
 from pymtl3 import *
 from pymtl3.passes.backends.verilog import *
+from pymtl3.passes.rtlir.rtype import RTLIRDataType as rdt
 from pymtl3.passes.testcases import (
     Bits32Foo,
     Bits32x5Foo,
@@ -59,6 +58,7 @@ from pymtl3.passes.testcases import (
     CaseDefaultBitsComp,
     CaseElifBranchComp,
     CaseFixedSizeSliceComp,
+    CaseForLoopEmptySequenceComp,
     CaseForRangeLowerUpperStepPassThroughComp,
     CaseHeteroCompArrayComp,
     CaseIfBasicComp,
@@ -327,7 +327,7 @@ CaseConnectPassThroughLongNameComp = set_attributes( CaseConnectPassThroughLongN
     ''',
     'REF_SRC',
     '''\
-        module DUT__a840bd1c84c05ea2
+        module DUT__2ae3b6d12e9855d1
         (
           input logic [0:0] clk,
           input logic [31:0] in_,
@@ -1170,6 +1170,34 @@ CaseNestedIfComp = set_attributes( CaseNestedIfComp,
     '''
 )
 
+CaseForLoopEmptySequenceComp = set_attributes( CaseForLoopEmptySequenceComp,
+    'REF_UPBLK',
+    '''\
+        always_comb begin : upblk
+          out = 4'd15;
+          for ( int unsigned i = 1'd0; i < 1'd0; i += 1'd1 )
+            out[2'(i)] = 1'd0;
+        end
+    ''',
+    'REF_SRC',
+    '''\
+        module DUT_noparam
+        (
+          input logic [0:0] clk,
+          output logic [3:0] out,
+          input logic [0:0] reset
+        );
+
+          always_comb begin : upblk
+            out = 4'd15;
+            for ( int unsigned i = 1'd0; i < 1'd0; i += 1'd1 )
+              out[2'(i)] = 1'd0;
+          end
+
+        endmodule
+    '''
+)
+
 CaseForRangeLowerUpperStepPassThroughComp = set_attributes( CaseForRangeLowerUpperStepPassThroughComp,
     'REF_UPBLK',
     '''\
@@ -2030,7 +2058,7 @@ CaseConnectConstStructAttrToOutComp = set_attributes( CaseConnectConstStructAttr
     ''',
     'REF_STRUCT',
     (
-        rdt.Struct('Bits32Foo', {'foo':rdt.Vector(32)}),
+        rdt.Struct(Bits32Foo, {'foo':rdt.Vector(32)}),
         dedent('''\
                   typedef struct packed {
                     logic [31:0] foo;
@@ -2069,10 +2097,10 @@ CaseConnectLiteralStructComp = set_attributes( CaseConnectLiteralStructComp,
     ''',
     'REF_STRUCT',
     (
-        rdt.Struct('NestedStructPackedPlusScalar', {
+        rdt.Struct(NestedStructPackedPlusScalar, {
           'foo':rdt.Vector(32),
           'bar':rdt.PackedArray([2], rdt.Vector(32)),
-          'woo':rdt.Struct('Bits32Foo', {'foo':rdt.Vector(32)}),
+          'woo':rdt.Struct(Bits32Foo, {'foo':rdt.Vector(32)}),
         }),
         dedent('''\
                   typedef struct packed {
@@ -2123,7 +2151,7 @@ CaseConnectArrayStructAttrToOutComp = set_attributes( CaseConnectArrayStructAttr
     ''',
     'REF_STRUCT',
     (
-        rdt.Struct('Bits32x5Foo', {'foo':rdt.PackedArray([5], rdt.Vector(32))}),
+        rdt.Struct(Bits32x5Foo, {'foo':rdt.PackedArray([5], rdt.Vector(32))}),
         dedent('''\
                   typedef struct packed {
                     logic [4:0][31:0] foo;
@@ -2168,10 +2196,10 @@ CaseConnectNestedStructPackedArrayComp = set_attributes( CaseConnectNestedStruct
     ''',
     'REF_STRUCT',
     (
-        rdt.Struct('NestedStructPackedPlusScalar', {
+        rdt.Struct(NestedStructPackedPlusScalar, {
           'foo':rdt.Vector(32),
           'bar':rdt.PackedArray([2], rdt.Vector(32)),
-          'woo':rdt.Struct('Bits32Foo', {'foo':rdt.Vector(32)}),
+          'woo':rdt.Struct(Bits32Foo, {'foo':rdt.Vector(32)}),
         }),
         dedent('''\
                   typedef struct packed {
