@@ -11,6 +11,7 @@ from pymtl3.passes.backends.verilog.util.utility import verilog_reserved
 from pymtl3.passes.rtlir import BehavioralRTLIRGenPass, BehavioralRTLIRTypeCheckPass
 
 from ....testcases import (
+    CaseBehavioralArraySubCompArrayStructIfcComp,
     CaseBits32ArraySubCompAttrUpblkComp,
     CaseBits32SubCompAttrUpblkComp,
 )
@@ -23,7 +24,7 @@ def run_test( case, m ):
 
   m.apply( BehavioralRTLIRGenPass( m ) )
   m.apply( BehavioralRTLIRTypeCheckPass( m ) )
-  upblks = m._pass_behavioral_rtlir_gen.rtlir_upblks
+  upblks = m.get_metadata( BehavioralRTLIRGenPass.rtlir_upblks )
   m_all_upblks = m.get_update_blocks()
   assert len(m_all_upblks) == 1
 
@@ -36,6 +37,7 @@ def run_test( case, m ):
     'case', [
       CaseBits32SubCompAttrUpblkComp,
       CaseBits32ArraySubCompAttrUpblkComp,
+      CaseBehavioralArraySubCompArrayStructIfcComp,
     ]
 )
 def test_yosys_behavioral_L5( case ):
